import math3d, physics
from core import *

class PlayerShot(object):
    def __init__(self, obj, pos, rots, attack,
                 attack_type, speed, age):
        self.obj=obj
        self.collider=self.collider=physics.Sphere(pos, max(self.obj.bb), 0.005)
        self.rots=rots

        self.attack=attack
        self.attack_type=attack_type
        self.speed=speed
        self.age=age

        self.cur_age=0
        self.dead=False

    def update(self):
        self.collider.push(math3d.move_forward_rot(self.rots))
        self.cur_age+=1
        if cur_age>=self.age:
            self.dead=True

    def render(self):
        glPushMatrix()
        glRotatef(*self.rots)
        glTranslatef(*self.collider.pos)
        self.obj.render()
        glPopMatrix()
        
class Player(object):
    def __init__(self, obj, shot_obj, pos, hitpoints=1,
                 speed=1, attack=1, attack_type='light',
                 defenses={'light':1,
                           'darkness':1,
                           'fire':1,
                           'water':1,
                           'plant':1,
                           'earth':1,
                           'energy':1},
                 projectile_speed=1,
                 attack_speed=1,
                 attack_dur=15,
                 color=[1,1,1,1]):
        self.obj=obj
        self.shot_obj=shot_obj
        self.collider=physics.Sphere(pos, max(self.obj.bb), 0.005)
        self.rotation=[0,0,0]

        self.color=color

        self.max_hitpoints=hitpoints
        self.hitpoints=int(self.max_hitpoints)
        self.speed=speed

        self.attack=attack
        self.attack_type=attack_type
        self.defenses=defenses
        self.projectile_speed=projectile_speed

        self.attack_speed=attack_speed
        self.cur_attack_phase=0

        self.attack_dur=attack_dur

        self.dead=False

        self.attackers=[]

    def get_attacks(self):
        a = list(self.attackers)
        self.attackers=[]
        return a

    def do_attack(self):
        self.cur_attack_phase+=1
        if self.cur_attack_phase==self.attack_speed:
            self.cur_attack_phase=0
            self.attacker.append(PlayerShot(self.shot_obj,
                                          self.pos,
                                          self.rotation,
                                          self.attack,
                                          self.attack_type,
                                          self.projectile_speed,
                                          self.attack_dur))
                              

    def update(self):
        self.collider.push([0,-0.0175,0])#gravity ;)
        self.collider.update()

        if self.collider.pos[1]-self.collider.get_height()<0:
            self.collider.pos[1]=self.collider.get_height()
            self.collider.velocity[1]=0
        if self.hitpoints<=0:
            self.dead=True

    def move(self, m):
        self.collider.push(m)

    def rotate(self, r):
##        self.rotation[0]+=r[0]#ignore x rotation ;)
        self.rotation[1]+=r[1]
        self.rotation[2]+=r[2]
        self.rotation=math3d.confine_rotation(self.rotation)

    def slow(self, m):
        self.sphere.pull(m)

    def check_collision(self, other):
        if isinstance(other, StaticPlatform):
            if self.collider.check_collision(other.collider):
                self.back_track()
                if self.collider.pos[1]>=other.collider.pos[1]:
                    self.collider.push([0,1,0])
                else:
                    self.collider.push([0,-1,0])
        if isinstance(other, Enemy):
            if self.collider.check_collision(other.collider):
                self.back_track()
        if isinstance(other, EnemyShot):
            other.dead=True
            self.hitpoints-=other.attack*self.defenses[other.attack_type]

    def render(self):
        glPushMatrix()
        glRotatef(self.rotation[0],1,0,0)
        glRotatef(self.rotation[1],0,1,0)
        glRotatef(self.rotation[2],0,0,1)
        glColor4f(*self.color)
        glTranslatef(*self.collider.pos)
        self.obj.render()
        glPopMatrix()
        

class StaticPlatform(object):
    def __init__(self, obj, pos):
        self.obj=obj
        self.collider=physics.BBox(pos, self.obj.bb, 0)

    def render(self):
        glPushMatrix()
        glTranslatef(*self.collider.pos)
        self.obj.render()
        glPopMatrix()

class EnemyShot(object):
    def __init__(self, obj, pos, rots, attack,
                 attack_type, speed, age):
        self.obj=obj
        self.collider=self.collider=physics.Sphere(pos, max(self.obj.bb), 0.005)
        self.rots=rots

        self.attack=attack
        self.attack_type=attack_type
        self.speed=speed
        self.age=age

        self.cur_age=0
        self.dead=False

    def update(self):
        self.collider.push(math3d.move_forward_rot(self.rots))
        self.cur_age+=1
        if cur_age>=self.age:
            self.dead=True

    def render(self):
        glPushMatrix()
        glRotatef(self.rotation[0],1,0,0)
        glRotatef(self.rotation[1],0,1,0)
        glRotatef(self.rotation[2],0,0,1)
        glTranslatef(*self.collider.pos)
        self.obj.render()
        glPopMatrix()

class Enemy(object):
    def __init__(self, obj, shot_obj, pos, los=15,
                 attack=1, attack_type="light",
                 attack_speed=0.01,
                 projectile_speed=0.1,
                 defenses={'light':1,
                           'darkness':1,
                           'fire':1,
                           'water':1,
                           'plant':1,
                           'earth':1,
                           'energy':1},
                 attack_dur=15,
                 color=[1,1,1,1]):
        self.obj=obj
        self.collider=physics.Sphere(pos, max(self.obj.bb), 0.005)
        self.big_sweeper=physics.Sphere(pos, los, 0.005)
        self.rotation=[0,0,0]

        self.color=color

        self.see_player=False

        self.attack=attack
        self.attack_type=attack_type
        self.attack_speed=attack_speed
        self.projectile_speed=projectile_speed

        self.defenses=defenses
        self.attack_dur=attack_dur

        self.cur_attack_phase=0

        self.attackers=[]

    def render(self):
        glPushMatrix()
        glColor4f(*self.color)
        glRotatef(self.rotation[0],1,0,0)
        glRotatef(self.rotation[1],0,1,0)
        glRotatef(self.rotation[2],0,0,1)
        glTranslatef(*self.collider.pos)
        self.obj.render()
        glPopMatrix()

    def get_attacks(self):
        a=list(self.attackers)
        self.attackers=[]
        return a

    def move(self, m):
        self.collider.push(m)
        self.big_sweeper.push(m)

    def update(self, player):
        if self.big_sweeper.check_collision(player.collider):
            self.see_player=True
        else:
            pass

        if self.see_player:
            self.rotation[0]=player.rotation[0]+180
            self.rotation=math3d.confine_rotation(self.rotation)
            self.do_attack()

    def do_attack(self):
        self.cur_attack_phase+=1
        if self.cur_attack_phase==self.attack_speed:
            self.cur_attack_phase=0
            self.attackers.append(EnemyShot(self.shot_obj,
                                          self.pos,
                                          self.rotation,
                                          self.attack,
                                          self.attack_type,
                                          self.projectile_speed,
                                          self.attack_dur))
    
