from core import *
import random

def randdec(minimum, maximum, pox):
    a = random.randint(int(minimum), int(maximum))
    a*=pox
    return a

def within(a, b, amount):
    if abs(a-b)<amount:return True
    return False

class ParticleBase(object):
    def render(self):
        glEnable(GL_BLEND)
        glDisable(GL_DEPTH_TEST)
        glPushMatrix()
        glTranslatef(*self.pos)
        glRotated(self.rotation[0],1,0,0)
        glRotated(self.rotation[1],0,1,0)
        glRotated(self.rotation[2],0,0,1)
        glColor4f(*self.color)

        glBindTexture(GL_TEXTURE_2D, self.image)
        #draw quads...
        glBegin(GL_QUADS)
        glVertex3f(-self.size, -self.size, 0)
        glTexCoord2f(0,0)
        
        glVertex3f(self.size, -self.size, 0)
        glTexCoord2f(0,1)
        
        glVertex3f(self.size, self.size, 0)
        glTexCoord2f(1,0)
        
        glVertex3f(-self.size, self.size, 0)
        glTexCoord2f(1,1)


        glVertex3f(-self.size, 0, -self.size)
        glTexCoord2f(0,0)
        
        glVertex3f(self.size, 0, -self.size)
        glTexCoord2f(0,1)
        
        glVertex3f(self.size, 0, self.size)
        glTexCoord2f(1,0)
        
        glVertex3f(-self.size, 0, self.size)
        glTexCoord2f(1,1)


        glVertex3f(0, -self.size, -self.size)
        glTexCoord2f(0,0)
        
        glVertex3f(0, self.size, -self.size)
        glTexCoord2f(0,1)
        
        glVertex3f(0, self.size, self.size)
        glTexCoord2f(1,0)
        
        glVertex3f(0, -self.size, self.size)
        glTexCoord2f(1,1)
        glEnd()

        glPopMatrix()
        glEnable(GL_DEPTH_TEST)
        glDisable(GL_BLEND)


class DirectionalParticle(ParticleBase):
    def __init__(self, pos, color, image, size, speed, age,
                 size_loss=0, rot_speed=[0,0,0]):
        self.pos=pos
        self.color=color
        self.image=image
        self.size=size
        self.speed=speed
        self.age=age
        self.rotation=[0,0,0]
        self.rot_speed=rot_speed

        self.size_loss=size_loss

        self.cur_age=0
        self.dead=False

    def update(self):
        self.cur_age+=1
        if self.cur_age>=self.age:
            self.dead=True
        self.pos[0]+=self.speed[0]
        self.pos[1]+=self.speed[1]
        self.pos[2]+=self.speed[2]
        self.size-=self.size_loss

        self.rotation[0]+=self.rot_speed[0]
        self.rotation[1]+=self.rot_speed[1]
        self.rotation[2]+=self.rot_speed[2]


class DirectionalGenerator(object):
    def __init__(self, generator_speed, domain, pos,
                 particle_color, particle_image,
                 particle_size, particle_speed,
                 particle_age, particle_size_loss=0,
                 particle_rot_speed=[0,0,0],
                 particle_speed_deviation=[1,1,1],
                 throw_particles=1):
        
        self.particles=[]
        self.speed=generator_speed
        self.domain=domain

        self.pos=pos
        self.throw_particles=throw_particles

        self.particle_color=particle_color
        self.particle_image=particle_image
        self.particle_size=particle_size
        self.particle_speed=particle_speed
        self.particle_age=particle_age
        self.particle_size_loss=particle_size_loss
        self.particle_rot_speed=particle_rot_speed

        self.particle_deviation=particle_speed_deviation

        self.step=0

    def get_start_pos(self):
        pos1=random.randint(-self.domain[0]*100, self.domain[0]*100)*0.01
        pos2=random.randint(-self.domain[1]*100, self.domain[1]*100)*0.01
        pos3=random.randint(-self.domain[2]*100, self.domain[2]*100)*0.01
        return [pos1+self.pos[0], pos2+self.pos[1], pos3+self.pos[2]]

    def get_start_speed(self):
        pos1=random.choice([self.particle_speed[0]-self.particle_deviation[0],
                            self.particle_speed[0],
                            self.particle_speed[0]+self.particle_deviation[0]])
        pos2=random.choice([self.particle_speed[1]-self.particle_deviation[1],
                            self.particle_speed[1],
                            self.particle_speed[1]+self.particle_deviation[1]])
        pos3=random.choice([self.particle_speed[2]-self.particle_deviation[2],
                            self.particle_speed[2],
                            self.particle_speed[2]+self.particle_deviation[2]])
        return [pos1, pos2, pos3]

    def update(self):
        self.step+=1
        if self.step >= self.speed:
            self.step=0
            for i in range(self.throw_particles):
                self.particles.append(DirectionalParticle(self.get_start_pos(),
                                               self.particle_color,
                                               self.particle_image,
                                               self.particle_size,
                                               self.get_start_speed(),
                                               self.particle_age,
                                               self.particle_size_loss,
                                               self.particle_rot_speed))
        for i in self.particles:
            if i.dead:
                self.particles.remove(i)
            i.update()

    def render(self):
        for i in self.particles:
            i.render()
                                           
